﻿using SoapEndpoint.Service.Proxy.Models.Flows;
using SoapEndpoint.Service.Proxy.Models.Systeem;
using SoapEndpoint.Service.Systeem.v1.LogbookService;

namespace SoapEndpoint.Tester.Scenario.Implementations.Schuldbemiddeling;

public class AutoReplyScenario : SchuldbemiddelingScenarioBase
{
    public Action<MessageboxBerichtModel> OnMessageReceived;
    public TimeSpan PollInterval = TimeSpan.FromSeconds(5);
    public override string Name => "Auto Reply Scenario";

    protected override async Task ExecuteAsync()
    {
        StartNewScenario(Name);

        while (true)
        {
            async Task HandleMessagesAsync(MessageboxBerichtinformatieModel info)
            {
                if (info?.Berichten.Any() == true)
                {
                    foreach (var message in info.Berichten) await SendReplyMessageAsync(message);
                }
            }

            if (Manager.ShvCertificate != null)
            {
                var info = await Manager.GetMessagesAsync<Berichttype>(Manager.ShvCertificate);
                await Manager.ConfirmMessagesAsync(Manager.ShvCertificate, info);
                await HandleMessagesAsync(info);
            }

            if (Manager.SeCertificate != null)
            {
                var info = await Manager.GetMessagesAsync<Berichttype>(Manager.SeCertificate);
                await Manager.ConfirmMessagesAsync(Manager.SeCertificate, info);
                await HandleMessagesAsync(info);
            }

            if (Manager.IsCancelled)
            {
                break;
            }

            await Task.Delay(PollInterval);
        }
    }

    private async Task SendReplyMessageAsync(MessageboxBerichtModel message)
    {
        OnMessageReceived?.Invoke(message);

        if (!Enum.TryParse(message.Berichttype, true, out Berichttype messageType))
        {
            throw new ArgumentOutOfRangeException(
                $"Message type {message.Berichttype} missing from {typeof(Berichttype)}");
        }

        var isShvMessage = message.Berichttype.StartsWith("SHV_");
        var isSeMessage = message.Berichttype.StartsWith("SE_");

        if (isSeMessage)
        {
            Manager.Se = new AdresboekOrganisatieModel
            {
                Referentienummer = message.Afzender.Value
            };
        }

        var conversation = new Conversation(message.ConversatieReferentienummer.Value, FlowType.Schuldbemiddeling);

        if (isSeMessage && Manager.ShvCertificate != null)
        {
            var replyType = GetShvReplyMessageType(messageType);
            if (replyType.HasValue)
            {
                await Manager.SHV_SendMessageAsync(conversation, replyType.Value);
            }
        }

        if (isShvMessage && Manager.SeCertificate != null)
        {
            var replyType = GetSeReplyMessageType(messageType);
            if (replyType.HasValue)
            {
                await Manager.SE_SendMessageAsync(conversation, replyType.Value,
                    new MessageModel<Berichttype>(message.Referentienummer, null, replyType.Value));
            }
        }
    }

    private static Berichttype? GetShvReplyMessageType(Berichttype input)
    {
        switch (input)
        {
            case Berichttype.SE_SaldoverzoekAntwoord:
                return Berichttype.SHV_Betaalvoorstel;
            case Berichttype.SE_BetaalvoorstelAntwoord:
            case Berichttype.SE_SaldoverzoekAntwoordStandaardAkkoord:
                return Berichttype.SHV_SchuldregelingGeaccordeerd;
            case Berichttype.SE_Statusverzoek: // TODO: This now only works for 'Standard' sector type.
                return Berichttype.SHV_Statusbericht;
            case Berichttype.SE_KlantGemeldVoorShvAntwoord:
                return Berichttype.SHV_Saldoverzoek;
            default:
                return default;
        }
    }

    private static Berichttype? GetSeReplyMessageType(Berichttype input)
    {
        switch (input)
        {
            case Berichttype.SHV_Saldoverzoek:
            case Berichttype.SHV_SaldoverzoekNuts:
            case Berichttype.SHV_SaldoverzoekPubliek:
            case Berichttype.SHV_SaldoverzoekZorg:
                return Berichttype.SE_SaldoverzoekAntwoord;
            case Berichttype.SHV_Betaalvoorstel:
                return Berichttype.SE_BetaalvoorstelAntwoord;
            case Berichttype.SHV_KlantGemeldVoorShv:
            case Berichttype.SHV_KlantBsnGemeldVoorShv:
                return Berichttype.SE_KlantGemeldVoorShvAntwoord;
            default:
                return default;
        }
    }
}